#include "register_types.h"
#include "enemy_world_state.hpp"
#include "entity_health.hpp"
#include "goap/state.hpp"
#include "nav_marker.hpp"
#include "nav_room.hpp"
#include "rts_actions.hpp"
#include "rts_game_mode.hpp"
#include "rts_player.hpp"
#include "rts_states.hpp"
#include "unit.hpp"
#include "unit_world_state.hpp"
#include "goap/action.hpp"
#include "goap/action_db.hpp"
#include "goap/actor_world_state.hpp"
#include "goap/goal.hpp"
#include "goap/planner.hpp"
#include "utils/register_types.hpp"
#include <gdextension_interface.h>
#include <godot_cpp/core/class_db.hpp>
#include <godot_cpp/core/defs.hpp>
#include <godot_cpp/godot.hpp>

namespace gd = godot;

void initialize_gdextension_types(gd::ModuleInitializationLevel p_level)
{
    if (p_level != gd::MODULE_INITIALIZATION_LEVEL_SCENE) {
        return;
    }
    utils::godot_cpp_utils_register_types();

    // always register actions before classes,
    // so that ActionDB::get_enum_hint is complete before _bind_methods
    goap::ActionDB::register_action<MoveToTarget>();
    goap::ActionDB::register_action<FireAtTarget>();
    goap::ActionDB::register_action<FindTarget>();
    goap::ActionDB::register_action<GetInMeleeRange>();
    goap::ActionDB::register_action<MeleeAttack>();
    goap::ActionDB::register_action<TankSelfHeal>();
    goap::ActionDB::register_action<TakeCover>();

    gd::ClassDB::register_class<goap::ActorWorldState>();
    gd::ClassDB::register_class<goap::Goal>();
    gd::ClassDB::register_class<goap::Planner>();
    gd::ClassDB::register_class<goap::State>();

    gd::ClassDB::register_class<MoveTo>();
    gd::ClassDB::register_class<Animate>();
    gd::ClassDB::register_class<Activate>();

    gd::ClassDB::register_class<UnitWorldState>();
    gd::ClassDB::register_class<EnemyWorldState>();
    gd::ClassDB::register_class<GoalMarker>();
    gd::ClassDB::register_class<Unit>();
    gd::ClassDB::register_class<RTSGameMode>();
    gd::ClassDB::register_class<RTSPlayer>();
    gd::ClassDB::register_class<EntityHealth>();
    gd::ClassDB::register_class<NavMarker>();
    gd::ClassDB::register_class<NavRoom>();
}

extern "C"
{
    // Initialization
    GDExtensionBool GDE_EXPORT metro_rts_library_init(GDExtensionInterfaceGetProcAddress p_get_proc_address, GDExtensionClassLibraryPtr p_library, GDExtensionInitialization *r_initialization)
    {
        gd::GDExtensionBinding::InitObject init_obj(p_get_proc_address, p_library, r_initialization);
        init_obj.register_initializer(initialize_gdextension_types);
        init_obj.set_minimum_library_initialization_level(gd::MODULE_INITIALIZATION_LEVEL_SCENE);

        return init_obj.init();
    }
}
